package lsedit;

/*
* @(#)FontChooser.java   1.2 98/07/31
*/

import java.lang.Integer;

// import java.util.Vector;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsEnvironment;
import java.awt.GridLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

/*
 * This applet displays a String with the user's selected 
 * fontname, style and size attributes.
*/

public class FontChooser extends JDialog implements ItemListener, ActionListener {
 
	class FontPanel extends JPanel {

		public FontPanel()
		{ 
		}

		public Dimension getPreferredSize()
		{
			return (new Dimension(m_fontPanelWidth, m_fontPanelHeight));
		}

		public Dimension getMinimumSize()
		{
			return(getPreferredSize());
		}

		public Dimension getMaximumSize()
		{
			return(getPreferredSize());
		}

		public void paintComponent (Graphics g) 
		{
			super.paintComponent(g);

			g.setColor(Color.darkGray);
			g.setFont(m_font);

			String change = "Set " + m_targetname + " font " + m_fontname + " ";
			int	   width;

			if (m_style < 4) {
				change += m_stylenames[m_style];
			} else {
				change += "style " + m_style;
			}
			change += " size " + m_size;
			width   = FontChooser.this.getWidth();

			Util.drawStringWrapped(g, change, m_fontPanelXMargin, m_fontPanelYMargin, width - 2 * m_fontPanelXMargin, m_fontPanelHeight - m_fontPanelYMargin, true, false);
	}	}
	  
	private final static int	  m_fontPanelWidth   = 150;
	private final static int	  m_fontPanelHeight  = 100;
	private final static int	  m_fontPanelXMargin =   5;
	private final static int	  m_fontPanelYMargin =  10;

	public  final static int	  TARGET_ALL           = 0;
	public  final static int	  TARGET_CLOSED        = 1;
	public  final static int	  TARGET_OPEN          = 2;
	public  final static int	  TARGET_OPEN_CLASS    = 3;
	public  final static int	  TARGET_CLIENTS       = 4;
	public  final static int	  TARGET_CARDINALS     = 5;
	public  final static int	  TARGET_RESULTS_TITLE = 6;
	public  final static int	  TARGET_RESULTS_TEXT  = 7;
	public  final static int	  TARGET_LEGEND_TITLE  = 8;
	public  final static int      TARGET_LEGEND_TEXT   = 9;
	public  final static int	  TARGET_QUERY_TITLE   = 10;
	public  final static int	  TARGET_QUERY_TEXT    = 11;
	public  final static int	  TARGET_TOC           = 12;
	public  final static int	  TARGET_UNDO          = 13;
	public  final static int      TARGET_MAP           = 14;
	public  final static int      TARGET_TITLE         = 15;
	public  final static int	  TARGET_TEXTBOX       = 16;
	public  final static int	  TARGET_FEEDBACK      = 17;
	public  final static int      TARGET_MENU          = 18;
	public  final static int	  TARGET_LAST          = 18;

	private final static String[] m_targetnames = new String[]
								{
									"ALL FONTS",
									"CLOSED LABEL",
									"OPEN LABEL",
									"OPEN CLASS LABEL",
									"CLIENT/SUPPIER LABEL",
									"CARDINALS",
									"RESULTS TITLE",
									"RESULTS TEXT",
									"LEGEND TITLE",
									"LEGEND TEXT",
									"QUERY TITLE",
									"QUERY TEXT",
									"TOC",
									"UNDO",
									"MAP",
									"TITLE",
									"TEXTBOX",
									"FEEDBACK",
									"MENU"
								};

	private final static String[] m_stylenames = new String[] 
								{
									"PLAIN",
									"BOLD",
									"ITALIC",
									"BOLD & ITALIC"
								};

	private final static String[] m_sizenames = new String[]
								{	
									"4", "5", "6", "7", "8", "9", "10", "12", "14", "16", "18", "20", "22", "24", "26", "28", "30"
								};

	private final static GraphicsEnvironment m_gEnv = GraphicsEnvironment.getLocalGraphicsEnvironment();
	private final static String m_envfonts[] = m_gEnv.getAvailableFontFamilyNames();


	private LandscapeEditorCore	m_ls;

	private	int			m_target;
	private String		m_targetname;
	private Font		m_font;
	private String		m_fontname;
	private int			m_style;
    private	int			m_size;

	private	FontPanel	m_fontPanel;
	private	JComboBox	m_targets, m_fonts, m_sizes, m_styles;
	private JButton		m_okButton, m_cancelButton, m_clearButton;

	protected void newFont(Font font)
	{
		m_font = font;

		m_fontname  = font.getFamily();
		m_style     = font.getStyle();
		m_size      = font.getSize();
	}

	protected void newFont()
	{
		newFont(Util.getCachedFont(m_fontname, m_style, m_size));
	}

	protected void setTarget(int index)
	{
		String	s;
		Font	font;
		int		i;

		switch (index) {
		case TARGET_ALL:
			font = Util.getCachedFont("Helvetica", Font.PLAIN, 12);
			s    = "***ALL***";
			break;
		case TARGET_OPEN_CLASS:	// Open Entity
			font = EntityInstance.getOpenClassFont();
			s    = "open class label";
			break;
		case TARGET_CLOSED:	// Closed Entity
			font = EntityInstance.getClosedFont();
			s    = "closed label";
			break;
		case TARGET_OPEN:	// Small Font
			font = EntityInstance.getSmallFont();
			s    = "small label";
			break;
		case TARGET_CARDINALS:	// Cardinals
			font = Cardinal.m_font;
			s    = "cardinal";
			break;
		case TARGET_CLIENTS:
			font = ClientSupplierSet.getClientSupplierFont();
			s    = "client/supplier";
			break;
		case TARGET_RESULTS_TITLE:
			font = ResultBox.getTitleFont();
			s    = "results title";
			break;
		case TARGET_RESULTS_TEXT:
			font = ResultBox.getTextFont();
			s    = "results text";
			break;
		case TARGET_LEGEND_TITLE:
			font = LegendBox.getTitleFont();
			s    = "legend title";
			break;
		case TARGET_LEGEND_TEXT:
			font = LegendBox.getTextFont();
			s    = "legend text";
			break;
		case TARGET_QUERY_TITLE:
			font = QueryBox.getTitleFont();
			s    = "query title";
			break;
		case TARGET_QUERY_TEXT:
			font = QueryBox.getTextFont();
			s    = "query text";
			break;
		case TARGET_TOC:
			font = TextTree.getTextFont();
			s    = "TOC";
			break;
		case TARGET_UNDO:
			font = HistoryBox.getTextFont();
			s    = "undo";
			break;
		case TARGET_MAP:
			font = MapBox.getTextFont();
			s    = "map";
			break;
		case TARGET_TITLE:
			font = TextBox.getTitleFont();
			s    = "title";
			break;
		case TARGET_TEXTBOX:
			font = TextBox.getTextFont();
			s    = "text box";
			break;
		case TARGET_FEEDBACK:
			font = Feedback.getTextFont();
			s    = "feedback";
			break;
		case TARGET_MENU:
			font = LandscapeEditorCore.getMenuFont();
			s    = "frame menu";
			break;
		default:
			System.out.println("FontChooser.setTarget Illegal index " + index);
			return;
		}
		
		m_target     = index;
		m_targetname = s;
		newFont(font);

		m_targets.setSelectedIndex(index);

		for (i = 0; i < m_envfonts.length; i++ ) {
			if (m_fontname.equalsIgnoreCase(m_envfonts[i])) {
				m_fonts.setSelectedIndex(i);
				break;
		}	}
        
		s = "" + m_size;

		for (i = 0; i < m_sizenames.length; i++ ) {
			if (s.equals(m_sizenames[i])) {
				m_sizes.setSelectedIndex(i);
				break;
		}	}

		if (m_style < m_stylenames.length) {
			m_styles.setSelectedIndex(m_style);
	}	}

	public FontChooser(JFrame frame, LandscapeEditorCore ls)
	{
		super(frame, "Select font for target", true /* modal */);

		Container	contentPane;
		JLabel		label;
		Font		font;
		int			i;

		m_ls        = ls;

		contentPane = getContentPane();
		font        = getFont().deriveFont(Font.BOLD);

        contentPane.setLayout( new BorderLayout() );

        JPanel topPanel          = new JPanel();
		JPanel targetPanel       = new JPanel();
        JPanel fontPanel         = new JPanel();
        JPanel sizePanel         = new JPanel();
        JPanel stylePanel        = new JPanel();
        JPanel sizeAndStylePanel = new JPanel();

        topPanel.setLayout( new BorderLayout() );

		targetPanel.setLayout( new GridLayout( 2, 1) );
        fontPanel.setLayout( new GridLayout( 2, 1 ) );
        sizePanel.setLayout( new GridLayout( 2, 1 ) );
        stylePanel.setLayout( new GridLayout( 2, 1 ) );
        
		sizeAndStylePanel.setLayout( new BorderLayout() );
        sizeAndStylePanel.add( BorderLayout.WEST, sizePanel );
        sizeAndStylePanel.add( BorderLayout.CENTER, stylePanel );

        topPanel.add( BorderLayout.WEST, targetPanel );
        topPanel.add( BorderLayout.CENTER, fontPanel );
        topPanel.add( BorderLayout.EAST, sizeAndStylePanel );

        contentPane.add( BorderLayout.NORTH, topPanel );

		{
			label = new JLabel();
			label.setText("Target");
			label.setFont(font);
			label.setHorizontalAlignment(JLabel.CENTER);
			targetPanel.add(label);
		}

		{
			label = new JLabel();
			label.setText("Fonts");
			label.setFont(font);
			label.setHorizontalAlignment(JLabel.CENTER);
			fontPanel.add(label);
		}

		{
			label = new JLabel();
			label.setText("Sizes");
			label.setFont(font);
			label.setHorizontalAlignment(JLabel.CENTER);
			sizePanel.add(label);
		}

		{
			label = new JLabel();
			label.setText("Styles");
			label.setFont(font);
			label.setHorizontalAlignment(JLabel.CENTER);
			stylePanel.add(label);
		}


		{
			m_targets = new JComboBox(m_targetnames);
			m_targetname = "target";

			m_targets.setSelectedIndex(0);
			m_targets.setMaximumRowCount( 9 );
			m_targets.addItemListener(this);
			targetPanel.add(m_targets);
		}

		{

			m_fonts = new JComboBox(m_envfonts);
			m_fonts.setMaximumRowCount( 9 );
			m_fonts.addItemListener(this);
        
			
			fontPanel.add(m_fonts);
		}

		{
			String	sizename = "" + m_size;

			m_sizes = new JComboBox(m_sizenames);
			m_sizes.setMaximumRowCount( 9 );
			m_sizes.addItemListener(this);
			sizePanel.add(m_sizes);
		}

		{
			m_styles = new JComboBox( m_stylenames );
			m_styles.setMaximumRowCount( 9 );
			m_styles.addItemListener(this);
			stylePanel.add(m_styles);
		}

        {
			m_fontPanel = new FontPanel();
			m_fontPanel.setBackground(Color.white);
			contentPane.add( BorderLayout.CENTER, m_fontPanel);
		}

		// Create an Okay button in a Panel; add the Panel to the window

		// Use a FlowLayout to center the button and give it margins.

		{
			JPanel bottomPanel = new JPanel();

			bottomPanel.setLayout(new FlowLayout(FlowLayout.CENTER, 15, 15));

			m_okButton = new JButton("OK");
			m_okButton.addActionListener(this);
			bottomPanel.add(m_okButton);

			m_clearButton = new JButton("Clear Cache");
			m_clearButton.addActionListener(this);
			bottomPanel.add(m_clearButton);

			m_cancelButton = new JButton("Cancel");
			m_cancelButton.addActionListener(this);
			bottomPanel.add(m_cancelButton);

			contentPane.add("South", bottomPanel);
		}

		setLocation(20, 20);
		setTarget(0);
		
		// Resize the window to the preferred size of its components
		pack();
		show();
    }

	public Font getFontChoice()
	{
		return (m_font);
	}

/*
 * ItemListener interface

 * Detects a state change in any of the Lists.  Resets the variable corresponding
 * to the selected item in a particular List.  Invokes changeFont with the currently
 * selected fontname, style and size attributes.
*/
    public void itemStateChanged(ItemEvent e) 
	{
        if ( e.getStateChange() == ItemEvent.SELECTED ) {

			Object	list  = e.getSource();

			if ( list == m_targets) {
				setTarget(m_targets.getSelectedIndex());
			} else if ( list == m_fonts ) {
				m_fontname = (String) m_fonts.getSelectedItem();
				newFont();
			} else if ( list == m_styles ) {
				m_style = m_styles.getSelectedIndex();;
				newFont();
			} else {
				String  sizename = (String) m_sizes.getSelectedItem();
				Integer newSize  = new Integer(sizename);
				m_size           = newSize.intValue();

				newFont();
			}
			m_fontPanel.repaint();
		}
    }

/* ActionListener interface */

	public void actionPerformed(ActionEvent e)
	{
		Object source = e.getSource();

		if (source != m_cancelButton) {
			if (source == m_clearButton) {
				Util.clearCachedFonts();
				JOptionPane.showMessageDialog(this, "The font cache has been cleared");
				return;
			}
			if (source != m_okButton) {
				return;
			}
			m_ls.setTargetFont(m_target, m_font);
		}
		setVisible(false);
	}
}



